package ru.pricepulse.parsingservice.ozon_parser.pool;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Profile("ozon")
public class WebDriverPool {

    private final Queue<WebDriver> availableDrivers = new ConcurrentLinkedQueue<>(); // Список доступных драйверов
    private final Queue<WebDriver> busyDrivers = new ConcurrentLinkedQueue<>(); // Список занятых драйверов
    private final ObjectFactory<WebDriver> webDriverFactory;

    public WebDriverPool(ObjectFactory<WebDriver> webDriverFactory) {
        this.webDriverFactory = webDriverFactory;
        int poolSize = 12;

        // Инициализация пула с указанным количеством драйверов
        for (int i = 0; i < poolSize; i++) {
            availableDrivers.add(createNewDriver());
        }
    }

    // Метод для создания нового экземпляра WebDriver
    private WebDriver createNewDriver() {
        return webDriverFactory.getObject();
    }

    // Метод для заимствования драйвера
    public WebDriver borrowDriver() {
        WebDriver driver = availableDrivers.poll(); // Получаем драйвер из доступных
        if (driver != null) {
            busyDrivers.add(driver); // Добавляем драйвер в занятые
        }
        return driver; // Возвращаем драйвер
    }

    // Метод для возврата драйвера в пул
    public void returnDriver(WebDriver driver) {
        busyDrivers.remove(driver); // Убираем драйвер из занятых
        availableDrivers.add(driver); // Возвращаем драйвер в доступные
    }

    // Метод для закрытия всех драйверов в пуле
    public void shutdownPool() {
        // Закрываем доступные драйверы
        for (WebDriver driver : availableDrivers) {
            driver.quit();
        }
        // Закрываем занятые драйверы
        for (WebDriver driver : busyDrivers) {
            driver.quit();
        }
        availableDrivers.clear();
        busyDrivers.clear();
    }

}
