package ru.pricepulse.parsingservice.ozon_parser.service.parsing;

import java.util.ArrayList;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Semaphore;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;
import ru.pricepulse.parsingservice.enumeration.Category;
import ru.pricepulse.parsingservice.ozon_parser.service.MarketplaceParsingService;

@Slf4j
@Service
@Profile("ozon")
public class OzonParsingService implements MarketplaceParsingService {

    private final AtomicBoolean stopFlag = new AtomicBoolean(false);
    private final ExecutorService pageExecutorService = Executors.newFixedThreadPool(5);
    private final Semaphore semaphore = new Semaphore(5); // Ограничиваем количество активных и ожидающих задач

    private final OzonCategoryPageParsingService categoryPageParsingService;

    public OzonParsingService(OzonCategoryPageParsingService categoryPageParsingService) {
        this.categoryPageParsingService = categoryPageParsingService;
    }

    public void processCategory(String url) {
        int pageIndex = 1;

        while (!stopFlag.get()) {
            try {
                semaphore.acquire(); // Получаем разрешение перед созданием новой задачи

                int finalPageIndex = pageIndex;
                String pageUrl = url + "&page=" + finalPageIndex;

                pageExecutorService.submit(() -> {
                    try {
                        categoryPageParsingService.parseCategoryPage(pageUrl, Category.LAPTOP, stopFlag);
                    } finally {
                        semaphore.release(); // Освобождаем разрешение после завершения задачи
                    }
                });

                ++pageIndex;

            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                break;
            }
        }

        if (stopFlag.get()) {
            log.info("Конец парсинга категории");
            pageExecutorService.shutdownNow();
        }
    }

}